/*!
 * Copyright (C) 2012 by Oleg Efimov
 *
 * See license text in LICENSE file
 */

/*global describe, it*/

var fs         = require('fs');
var assert     = require('assert');
var difflet    = require('difflet')({indent: 2, comment: true});
var proto2json = require(process.env.LIB_COV ? '../lib-cov/proto2json' : '../lib/proto2json');

// Patch assert.deepEqual
var originalDeepEqual = assert.deepEqual;

assert.deepEqual = function (actual, expected, message) {
  try {
    originalDeepEqual.apply(null, arguments);
  } catch (err) {
    var diff = difflet.compare(expected, actual);
    err.toString = function () { return 'AssertionError: ' + diff; };
    throw err;
  }
};

describe('proto.jison', function () {
  it('should be parsable', function () {
    var jison_source = fs.readFileSync(__dirname + '/../src/proto.jison', 'utf8');

    assert.doesNotThrow(function () {
      require("ebnf-parser").parse(jison_source);
    });
  });
});

describe('proto2json', function () {
  describe('should parse and return in sync mode', function () {
    it('well-formed proto', function (done) {
      var result = proto2json.parse("package Test;");

      assert.deepEqual(
        result,
        {
          "package": "Test",
          "messages": {}
        }
      );

      done();
    });

    it('wrong proto', function (done) {
      var result = proto2json.parse("package 1234;");

      assert.ok(result instanceof Error);

      assert.ok(result.toString().indexOf("Error: Parse error on line 1:") !== -1);

      done();
    });
  });

  describe('should parse well-formed proto files', function () {
    var files = fs.readdirSync(__dirname + '/fixtures/positive')
      .filter(function (file) { return file.match(/\.proto$/); })
      .sort();

    files.forEach(function (file) {
      var name = file.replace(/\.proto$/, '');

      var proto = fs.readFileSync(__dirname + '/fixtures/positive/' + name + '.proto', 'utf8');
      var json = fs.readFileSync(__dirname + '/fixtures/positive/' + name + '.json', 'utf8');

      json = JSON.parse(json);

      it(name + '.proto', function (done) {
        proto2json.parse(proto, function (err, parse_result) {
          if (err) {
            throw err;
          }

          assert.deepEqual(parse_result, json);

          done();
        });
      });
    });
  });

  describe('should fails on wrong proto files', function () {
    var files = fs.readdirSync(__dirname + '/fixtures/negative')
      .filter(function (file) { return file.match(/\.proto$/); })
      .sort();

    files.forEach(function (file) {
      var name = file.replace(/\.proto$/, '');

      var proto = fs.readFileSync(__dirname + '/fixtures/negative/' + name + '.proto', 'utf8');
      var error = fs.readFileSync(__dirname + '/fixtures/negative/' + name + '.error', 'utf8');

      it(name + '.proto', function (done) {
        proto2json.parse(proto, function (err) {
          if (!err) {
            throw err;
          }

          assert.ok(err.toString().indexOf(error) !== -1);

          done();
        });
      });
    });
  });
});
